import itertools
import wrapio
import os
import functools

from . import api
from . import helpers
from . import utils

from . import _colors


__all__ = ('finish', 'update', 'input', 'password', 'accept', 'reject',
           'question', 'confirm', 'select', 'traverse', 'path')


def finish():

    """
    Stop listening for keypresses.
    """

    api.finish(False)


def update(value):

    """
    Update hint.
    """

    api.update(value)


_default_color = _colors.info


def _input(*args, **kwargs):

    result = api.edit(*args, **kwargs)

    return result


def input(*args, color = _default_color, erase = False, view = None, **kwargs):

    """
    input(*args, color=blue, erase=False, view=None, **kwargs)

    Await and return input.

    :param str color:
        Paints output.
    :param bool erase:
        Same as :func:`.api.edit`\'s ``full``.
    :param func view:
        Called with ``(result)``; should return ``list[str]`` to output.

    Other arguments are passed to :func:`.api.edit`.

    .. code-block:: py

        answer = survey.input('How old are you? ', '(age) ')
        print(f'You are {answer} years old!')
    """

    result = _input(*args, **kwargs)

    shows = None if view is None else view(result)

    api.respond(shows = shows, color = color, full = erase)

    return result


def password(*args, rune = '*', color = '\x1b[90m', **kwargs):

    """
    password(*args, rune='*', color=grey, **kwargs)

    Await and return input. Uses ``rune`` to replace keypresses.

    ``color`` (grey) and other arguments are passed to :func:`.input`.

    .. code-block:: py

        answer = survey.password('Secret: ')
        print(f'Singing in with {answer}...')
    """

    ofunnel = kwargs.pop('funnel', None)
    nfunnel = lambda _: rune

    funnel = helpers.combine_functions(ofunnel, nfunnel)

    def view(value):
        return (len(value) * rune,)

    result = input(*args, view = view, **kwargs, color = color, funnel = funnel)

    return result


def accept():

    """
    Respond with green.
    """

    api.respond(color = _colors.done)


def reject():

    """
    Respond with red.
    """

    api.respond(color = _colors.fail)


def question(*args, **kwargs):

    """
    Await and return input. Use :func:`.accept` or :func:`.reject` to respond.

    Other arguments are passed to :func:`.api.edit`.

    .. code-block:: py

        check = lambda answer: answer.isdigit()
        answer = survey.question('Solve 2+2: ', check = check)
        correct = int(answer) == 4
        (survey.accept if correct else survey.reject)()
    """

    return api.edit(*args, **kwargs)


def confirm(*args,
            default = None,
            sentiments = [
                ('n', 'no', '0', 'false', 'f'),
                ('y', 'yes', '1', 'true', 't')
            ],
            responses = ('No', 'Yes'),
            **kwargs):

    """
    confirm(*args, default=None, sentiments=[['n','no','0','false','f'],['y','yes','1','true','t']], responses=('No','Yes'), **kwargs)

    Await sentiment input and return respective :func:`.bool` value.

    :param list[set,set] sentiments:
        Containers of negative and positive options.
    :param list[str,str] responses:
        Negative and positive responses.
    :param bool default:
        Match empty responses to this.

    When ``hint`` is ommited, a suitable one takes its place.

    Other arguments except ``view``, ``limit`` and ``check`` are passed to
    :func:`.input`.

    .. code-block:: py

        result = survey.confirm('Proceed? ', default = True)
        print('Doing...' if result else 'Skipping...')
    """

    if not 'hint' in kwargs:
        template = helpers.paint('({0}/{1}) ', _colors.hint)
        color = kwargs.get('color', _default_color)
        kwargs['hint'] = utils.hint.confirm(template, default, color = color)

    index = None
    def check(value):
        if value:
            nonlocal index
            value = value.lower()
            for (index, sentiment) in enumerate(sentiments):
                if not value in sentiment:
                    continue
                break
            else:
                return False
        elif default is None:
            return False
        return True

    limit = max(map(len, itertools.chain.from_iterable(sentiments)))

    value = None
    def view(_):
        nonlocal value
        value = default if index is None else bool(index)
        response = responses[value]
        return (response,)

    input(
        *args,
        **kwargs,
        view = view,
        limit = limit,
        check = check
    )

    return value


def _select_hint_template_instructs(multi):

    values = ['filter: type', 'move: ↑↓']

    if multi:
        values.extend(('pick: → all: →→', 'unpick: ← all: ←←'))

    return values


def _select_hint_template(instructs):

    template = '{0} [' + ' | '.join(instructs) + ']'

    return template


def _select_hint(template, callback, external = True):

    template = helpers.paint(template, _colors.hint)

    (invoke, hint) = utils.hint.select(template, external = external)

    callback = helpers.succeed_functions(invoke, callback)

    return (callback, hint)


def _select(*args,
            color = _default_color,
            erase = False,
            **kwargs):

    if not 'hint' in kwargs:
        multi = kwargs.get('multi', False)
        instructs = _select_hint_template_instructs(multi)
        template = _select_hint_template(instructs)
        kwargs['hint'] = template

    template = kwargs['hint']
    if not template is None:
        callback = kwargs.get('callback')
        (invoke, hint) = _select_hint(template, callback)
        kwargs['callback'] = invoke
        kwargs['hint'] = hint

    result = api.select(*args, **kwargs, color = color)

    return result


def select(*args, color = _default_color, erase = False, view = None, **kwargs):

    """
    select(*args, color=blue, erase=False, view=None, **kwargs)

    Await and return input.

    :param str color:
        Paints output.
    :param bool erase:
        Same as :func:`.api.select`\'s ``full``.
    :param func view:
        Called with ``(result)``; should return ``list[str]`` to output.

    ``focus`` as ``color`` and other arguments are passed to
    :func:`.api.select`.

    When ``hint`` is ommited, a suitable one takes its place.

    Filter value is formatted on ``hint``\'s first placeholder (``{0}``).

    .. code-block:: py

        options = ('eat', 'sleep', 'code', 'repeat')
        index = survey.select(options, 'Do: ')

        options = ('bacon', 'lettuce', 'tomato', 'malted bread')
        indexes = survey.select(options, 'Use: ', multi = True)
    """

    try:
        kwargs['color'] = kwargs.pop('focus')
    except KeyError:
        pass

    result = _select(*args, **kwargs)

    shows = None if view is None else view(result)

    api.respond(shows = shows, color = color, full = erase)

    return result


def _traverse(path, show, able, next, *args, look = False, **kwargs):

    (options, displays) = next(path)

    track = wrapio.Track()

    wall = not len(path) > 1
    back = None

    @track.call
    def move_x(index, left, size):
        nonlocal back
        if left:
            if wall:
                raise api.Abort()
        else:
            option = options[index]
            if not able(path, option):
                raise api.Abort()
        back = left
        api.finish(True)

    try:
        template = kwargs.pop('hint')
    except KeyError:
        instructs = _select_hint_template_instructs(False)
        if any(able(path, option) for option in options):
            instructs.append('next: →')
        if not wall:
            instructs.append('back: ←')
        template = _select_hint_template(instructs)

    if show:
        showpath = show(path)
        template = template.format(showpath, '{0}')

    check = kwargs.pop('check', None)

    def subcheck(index):
        if back is None:
            option = options[index]
            return not check or check(path, option)
        return True

    index = _select(
        displays,
        *args,
        **kwargs,
        hint = template,
        check = subcheck,
        callback = track.invoke
    )

    if back is True:
        path.pop()
    else:
        option = options[index]
        path.append(option)
        if back is None:
            return path

    api.respond(shows = (), full = True)

    return _traverse(path, show, able, next, *args, check = check, **kwargs)


def traverse(initial, show, able, next, *args, **kwargs):

    """
    traverse(initial, show, able, next, *args, **kwargs)

    Cycle through proceedural pages of options.

    :param object initial:
        Used on ``next`` to determine options.
    :param func show:
        Called with ``(path)``; should return :class:`str` for hint formatting.
    :param func able:
        Called with ``(path, option)``; should return :class:`bool` for
        whether advancement is possible.
    :param func next:
        Called with ``(path)``; should return ``(options, displays)`` with
        latter used for drawing and former for selecting.
    :param bool look:
        Whether to assess if any option is advancable.

    ``show`` result is formatted on first placeholder (``{0}``) and filter value
    on second (``{1}``).

    ``check`` takes ``(path, option)``.
    """

    if 'multi' in kwargs:
        raise ValueError('multi makes no sense')

    view = kwargs.pop('view', None)
    color = kwargs.pop('color', _default_color)
    erase = kwargs.pop('erase', False)

    path = [initial]

    _traverse(path, show, able, next, *args, **kwargs)

    shows = None if view is None else view(path)

    api.respond(shows = shows, color = color, full = erase)

    return path


_path_units = ('f', '/', 'd')
_path_units = ('(' + unit + ')' for unit in _path_units)
_path_units = (helpers.paint(unit, _colors.hint) for unit in _path_units)
_path_units = (next(_path_units), (next(_path_units), next(_path_units)))

def path(directory,
         *args,
         units = _path_units,
         able = None,
         look = True,
         **kwargs):

    """
    path(directoy, *args, units=['f',['/','d']], able=None, look=True, **kwargs)

    Traverse a system directory until a file is selected.

    :param tuple[str,tuple[str,str]] units:
        ``f`` for non-dirs and ``/`` for empty dirs, ``d`` for dirs.
    :param func able:
        Called with ``(path)`` and should return :class:`bool` for whether to
        include in options.
    :param bool look:
        Works same as in :func:`.traverse`, but changed using memoization.

    ``check`` takes ``(path)``
    """

    def make(trail, *rest):
        return os.path.join(*trail, *rest)

    color = kwargs.get('color', _default_color)

    def show(trail):
        path = make(trail)
        path = helpers.paint(path, color)
        return path

    @functools.lru_cache(None)
    def contents(path):
        names = os.listdir(path)
        stick = lambda name: os.path.join(path, name)
        paths = map(stick, names)
        if able:
            try:
                (paths, names) = helpers.multifilter(able, paths, names)
            except ValueError:
                paths = names = ()
        else:
            paths = tuple(paths)
        return (names, paths)

    def able_(trail, part):
        path = make(trail, part)
        return os.path.isdir(path) and all(contents(path))

    def next(trail):
        path = make(trail)
        (names, paths) = contents(path)
        limit = max(map(len, names))
        shows = []
        for (name, path) in zip(names, paths):
            pred = os.path.isdir(path)
            unit = units[pred]
            if pred:
                pred = all(contents(path))
                unit = unit[pred]
            push = limit - len(name) + 1
            show = name + ' ' * push + unit
            shows.append(show)
        return (names, shows)

    try:
        check = kwargs['check']
    except KeyError:
        pass
    else:
        def subcheck(trail, part):
            path = make(trail, part)
            return check(path)
        kwargs['check'] = subcheck

    def view(trail):
        path = make(trail)
        return (path,)

    directory = os.path.join(directory, '') # ensure "/"

    trail = traverse(
        directory,
        show,
        able_,
        next,
        *args,
        **kwargs,
        look = look,
        view = view
    )

    result = make(trail)

    return result

del _path_units



import termios

from . import helpers


__all__ = ('IO',)


class IO:

    __slots__ = ('_i', '_o', '_buffer', '_fd', '_save', '_mode', '_block',
                 '_atomic')

    def __init__(self, i, o):

        self._i = i
        self._o = o

        self._buffer = []

        self._fd = i.fileno()

        self._save = None
        self._mode = None

        self._block = None

        self._atomic = helpers.Atomic(self.start, self.stop)

    @property
    def atomic(self):

        return self._atomic

    def _swap(self, mode):

        termios.tcsetattr(self._fd, termios.TCSAFLUSH, mode)

    def _fix(self, block):

        self._block = block

        self._mode[6][termios.VMIN] = int(block)

        self._swap(self._mode)

    def start(self):

        """
        Save settings and enter cbreak mode.
        """

        self._save = termios.tcgetattr(self._fd)

        mode = termios.tcgetattr(self._fd)

        mode[3] &= ~(termios.ECHO | termios.ECHONL | termios.ICANON)
        mode[6][termios.VTIME] = 0

        self._mode = mode

        self._fix(True)

    def stop(self):

        """
        Restore initial settings.
        """

        self._swap(self._save)

        self._save = None

    def send(self, value):

        """
        Write to output buffer.
        """

        self._o.write(value)
        self._o.flush()

    def recv(self, block = True):

        """
        Read from input buffer.
        """

        if not block is self._block:
            self._fix(block)

        try:
            return self._buffer.pop(0)
        except IndexError:
            pass

        return self._i.read(1)

    def feed(self, data):

        """
        Funnel additional data to be read before input buffer.
        """

        self._buffer.extend(data)

    def ring(self):

        """
        Sound system bell.
        """

        self.send('\a')



import types
import re

from . import helpers


__all__ = ('Cursor',)


_CSI = '\x1b'


ClearMode = types.SimpleNamespace(right = 0, left = 1, full = 2, extend = 3)


EraseMode = types.SimpleNamespace(right = 0, left = 1, full = 2)


class Cursor:

    __slots__ = ('_io', '_hidden')

    def __init__(self, io):

        self._io = io

        self._hidden = helpers.Atomic(self.hide, self.show)

    @property
    def hidden(self):

        return self._hidden

    def _send(self, code, *args, private = False):

        check = lambda value: not value is None
        params = ';'.join(map(str, filter(check, args)))
        if args:
            params = '[' + params

        self._io.send(_CSI + params + code)

    def up(self, size = None):

        """
        Move `size` cells up.
        """

        if size == 0:
            return

        self._send('A', size)

    def down(self, size = None):

        """
        Move `size` cells down.
        """

        if size == 0:
            return

        self._send('B', size)

    def left(self, size = None):

        """
        Move `size` cells left.
        """

        if size == 0:
            return

        self._send('D', size)

    def right(self, size = None):

        """
        Move `size` cells right.
        """

        if size == 0:
            return

        self._send('C', size)

    def goto(self, x = None):

        """
        Move to `x` column.
        """

        self._send('G', x)

    def next(self, size = None):

        """
        Move to beginning of `size` lines down.
        """

        if size == 0:
            self.goto(0)
            return

        self._send('E', size)

    def last(self, size = None):

        """
        Move to beginning of `size` lines up.
        """

        if size == 0:
            self.goto(0)
            return

        self._send('F', size)

    def move(self, y, x):

        """
        Move to `x` and `y` coordinates:
        """

        self._send('f', y, x)

    def clear(self, mode = None):

        """
        Clear display.
        """

        self._send('J', mode)

    def erase(self, mode = None):

        """
        Erase in-line.
        """

        self._send('K', mode)

    def save(self):

        """
        Save current location.
        """

        self._send('7')

    def load(self):

        """
        Move to saved location.
        """

        self._send('8')

    def show(self):

        """
        Show.
        """

        self._send('h', '?25')

    def hide(self):

        """
        Hide.
        """

        self._send('l', '?25')

    def _locate(self, pattern = re.compile(_CSI + '\[(\d+);(\d+)R$')):

        buffer = ''
        funnel = []

        self._send('n', '6')

        while True:
            rune = self._io.recv()
            buffer += rune
            if rune == 'R':
                index = buffer.rindex(_CSI)
                portion = buffer[index:]
                match = pattern.match(portion)
                if match:
                    break
                else:
                    funnel.extend(buffer)

        groups = match.groups()
        values = (groups[index] for index in range(2))

        return tuple(map(int, values))

    def locate(self):

        """
        Get current location.
        """

        with self._io.atomic:
            result = self._locate()

        return result

    def measure(self):

        """
        Get screen size.
        """

        self.save()

        with self._hidden:
            self.move(999, 9999)
            size = self.locate()
            self.load()

        return size



import sys
import types
import os
import collections

from . import io
from . import cursor
from . import bases
from . import tools
from . import helpers


__all__ = ('update', 'respond', 'edit', 'select')


_io = io.IO(sys.stdin, sys.stdout)


_cursor = cursor.Cursor(_io)


_assets = types.SimpleNamespace(
    caption = tools.Caption(
        tools.Display(
            _io,
            _cursor
        )
    ),
    machine = None,
    shows = [],
    fall = 0,
    signal = None,
    gentle = False,
)


def _finish(gentle):

    _assets.signal()

    _assets.gentle = gentle


def finish(gentle):

    """
    Stop listening for keypresses.
    """

    _finish(gentle)


def _update(value):

    _assets.machine.clear()

    _assets.caption.update(value)

    _assets.machine.draw()
    _assets.machine.focus()


def update(value):

    """
    Update hint.
    """

    with _cursor.hidden:
        _update(value)


def _respond(shows, full, color, draw, delimit):

    _assets.machine.clear()

    if shows is None:
        shows = _assets.shows.copy()

    _assets.shows.clear()

    if color:
        paint = lambda value: helpers.paint(value, color)
        shows = map(paint, shows)

    show = delimit.join(shows)

    if not full:
        show = _assets.fall * os.linesep + show

    _assets.caption.finish(show, full = full)

    if not full:
        _io.send(os.linesep)


def respond(shows = None,
            full = False,
            color = None,
            draw = True,
            delimit = ', '):

    """
    Reset state and show results.

    :param list[str] show:
        What to show instead.
    :param bool full:
        Whether to also erase prompt.
    :param str color:
        Used to paint results.
    :param bool draw:
        Whether to show results.
    :param str delimit:
        Used to join results together.
    """

    with _cursor.hidden:
        _respond(shows, full, color, draw, delimit)


def _execute(machine, check):

    _assets.machine = machine

    translator = bases.Translator(_io, callback = _assets.machine.invoke)

    source = bases.Source(_io, callback = translator.invoke)

    _assets.signal = source.done

    while True:
        source.stream()
        result = _assets.machine.get()
        if _assets.gentle and check and not check(result):
            _io.ring()
            continue
        break

    shows = machine.view(result)
    _assets.shows.extend(shows)

    return result


def _measure():

    (my, mx) = _cursor.measure()

    my -= 1
    mx -= 1

    _assets.caption.locate(mx)

    return (my, mx)


class Abort(Exception):

    __slots__ = ()


def _callback(function):

    def callback(name, *args):
        if function:
            result = _assets.machine.get()
            try:
                function(name, result, *args)
            except Abort:
                _io.ring()
                return
        if name == 'submit':
            _finish(True)

    return callback


def _line_edit_single(my, mx, limit, funnel, callback):

    editor = tools.LineEditor(
        _io,
        _cursor,
        mx,
        limit,
        funnel,
        callback = callback
    )

    return editor


def _line_edit_multi(my, mx, trail, limit, funnel, callback):

    def finchk():
        init = len(editor.subs)
        subs = editor.subs[-trail:]
        result = len(subs) == trail and not any(sub.buffer for sub in subs)
        if result:
            extra = init == trail
            editor.delete(True, trail - extra)
        return result

    editor = tools.MultiLineEditor(
        _io,
        _cursor,
        finchk,
        my,
        mx,
        limit,
        funnel,
        callback = callback
    )

    return editor


def edit(prompt = None,
         *,
         hint = None,
         limit = None,
         funnel = None,
         trail = 2,
         check = None,
         callback = None,
         multi = False):

    """
    Await and return input from user.

    :param str prompt:
        Persistent prompt shown before input.
    :param str hint:
        Temporary grayed-out prompt shown after prompt.
    :param int limit:
        Max allowed size of internal rune buffer.
    :param func funnel:
        Used with ``(rune)`` and must return some rune.
    :param int trail:
        Only with ``multi``. Amount of empty lines required to submit.
    :param func check:
        Used with ``(result)`` for validation and must return :class:`bool`.
    :param func callback:
        Used with ``(name, *args)`` for listening to keypress events.
    :param bool multi:
        Whether to accept line breaks.

    Event names are followed by current result, and then arguments.
    """

    (my, mx) = _measure()

    callback = _callback(callback)

    if multi:
        machine = _line_edit_multi(my, mx, trail, limit, funnel, callback)
        fall = 1
    else:
        machine = _line_edit_single(my, mx, limit, funnel, callback)
        fall = 0

    _assets.fall = fall

    _assets.caption.create(prompt or '', hint or '', fall = multi)

    value = _execute(machine, check)

    return value


def _select_single(my, mx, options, prefix, indent, funnel, filter, callback):

    select = tools.Select(
        _io,
        _cursor,
        my,
        mx,
        options,
        prefix,
        indent,
        funnel,
        filter,
        callback = callback
    )

    return select


def _select_multi(my, mx,
                  options,
                  prefix,
                  indent,
                  funnel,
                  filter,
                  unpin,
                  pin,
                  indexes,
                  callback):

    select = tools.MultiSelect(
        unpin,
        pin,
        indexes,
        _io,
        _cursor,
        my,
        mx,
        options,
        prefix,
        indent,
        funnel,
        filter,
        callback = callback
    )

    return select


def _select_filter(pairs, argument):

    argument = argument.lower()

    counter = collections.Counter(argument)

    for (index, option) in pairs:
        option = option.lower()
        for (rune, rcount) in counter.items():
            ccount = option.count(rune)
            if 0 < ccount >= rcount:
                continue
            break
        else:
            yield (index, option)


def select(options,
           prompt = None,
           *,
           hint = None,
           prefix = '> ',
           color = None,
           indent = None,
           funnel = None,
           filter = None,
           limit = 6,
           unpin = '[ ] ',
           pin = '[X] ',
           indexes = (),
           check = None,
           callback = None,
           multi = False):

    """
    Draw a menu of options. Traverse using **↑** and **↓** keys. Type to filter.
    Enter to submit. Return index(es) of option(s).

    This should not be used directly for most purposes.

    :param str prompt:
        Persistent prompt shown before input.
    :param str hint:
        Temporary grayed-out prompt shown after prompt.
    :param str prefix:
        Indicator for unpinned item.
    :param str color:
        ANSI color sequence to paint unpinned item.
    :param func funnel:
        Used with current ``(option)`` and must return some :class:`str`.
    :param int limit:
        Max amount of options displayed at any point.
    :param str unpin:
        Indicator for un-selected items (multi only).
    :param str pin:
        Indicator for selected items (multi only).
    :param list[int] indexes:
        Indexes options to pre-select (multi only).
    :param func check:
        Used with ``(result)`` for validation and must return :class:`bool`.
    :param func callback:
        Used with ``(name, *args)`` for listening to keypress events.
    :param bool multi:
        Whether to allow multiple selections using **←** and **→** keys.

    Event names are followed by current result, and then arguments.
    """

    (my, mx) = _measure()

    if color:
        def paint(index, option):
            return helpers.paint(option, color)
        funnel = helpers.combine_functions(funnel, paint, index = 1)

    if not limit:
        limit = len(options)
    my = min(my, limit)

    pushers = [len(prefix)]
    if indent:
        pushers.append(indent)
    cover = max(pushers)
    if multi:
        cover += max(map(len, (unpin, pin)))

    mx -= cover

    callback = _callback(callback)

    if not filter:
        filter = _select_filter

    if multi:
        machine = _select_multi(
            my, mx,
            options,
            prefix,
            indent,
            funnel,
            filter,
            unpin,
            pin,
            indexes,
            callback
        )
    else:
        machine = _select_single(
            my, mx,
            options,
            prefix,
            indent,
            funnel,
            filter,
            callback
        )

    _assets.fall = 0

    _assets.caption.create(prompt or '', hint or '', fall = 1)

    machine.draw()
    machine.focus()

    with _cursor.hidden:
        value = _execute(machine, check)

    return value



import setuptools

with open('README.rst') as file:
    readme = file.read()

name = 'survey'

version = '2.1.0'

author = 'Exahilosys'

url = f'https://github.com/{author}/{name}'

setuptools.setup(
    name = name,
    python_requires = '>=3.5',
    version = version,
    url = url,
    packages = setuptools.find_packages(),
    license = 'MIT',
    description = 'A simple library for creating beautiful interactive prompts.',
    long_description = readme,
    install_requires = [
        'wrapio>=0.3.5'
    ],
    extras_require = {
        'docs': [
            'sphinx',
            'sphinx_rtd_theme'
        ]
    }
)



import wrapio

from . import api
from . import helpers

from . import _colors


__all__ = ('hint',)


class hint:

    def confirm(template,
                default = None,
                options = ('n', 'y'),
                inflate = True,
                color = _colors.info):

        """
        Create a hint for :func:`confirm` tool.

        :param str template:
            Should include 2 indexed formatting placeholders.
        :param bool default:
            Use to indicate if any option should be highlighted.
        :param list[str] options:
            2-length list containing negative and positive option respectively.
        :parm bool inflate:
            Whether to use :meth:`str.upper` on highlighted option.
        :param str color:
            Used for highlighting.
        """

        options = list(options)

        if not default is None:
            index = int(default)
            option = options[index]
            if inflate:
                option = option.upper()
            if color:
                option = helpers.paint(option, color)
            options[index] = option
            if default is True:
                options = reversed(options)

        result = template.format(*options)

        return result

    def select(template,
               default = 'type',
               external = False,
               color = _colors.info):

        """
        Create a callback and hint for :func:`select` tool.

        :param str template:
            Should include up to one formatting placeholder for filter value.
        :param str default:
            Used when filter value is empty instead.
        :param bool external:
            Whether ``default`` should be used at all.
        :param str color:
            Used for highlighting filter value.

        Use the callback for automatically updating hint.
        """

        def choose(value):
            if not value:
                return '' if external else default
            return helpers.paint(value, color)

        def format(value):
            show = choose(value)
            return template.format(show)

        track = wrapio.Track()

        @track.call
        def filter(result, value):
            hint = format(value)
            api.update(hint)

        hint = format('')

        return (track.invoke, hint)
